"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.customAxios = exports.customFetch = void 0;
const axios_1 = require("axios");
const Config_1 = require("../Config");
const node_fetch_1 = require("node-fetch");
const lib_types_1 = require("@shardeum-foundation/lib-types");
/**
 * A custom fetch function with a max response body size limit.
 * @param input URL or Request object.
 * @param init Optional fetch configuration.
 * @param maxBytes Maximum response size in bytes (default: config.maxResponseSize).
 * @returns A Response.
 */
async function customFetch(input, init, maxBytes) {
    const downloadLimit = maxBytes ?? Config_1.config.maxResponseSize;
    return await (0, node_fetch_1.default)(input, { ...init, size: downloadLimit });
}
exports.customFetch = customFetch;
/**
 * Creates a custom axios instance with size limiting
 * @param maxBytes Maximum response size in bytes (default: 10MB)
 * @param axiosConfig Additional axios axiosConfig options
 * @returns Custom axios instance
 */
function customAxios(maxBytes, axiosConfig = {}) {
    // Use the provided maxBytes or fall back to config.maxResponseSize
    const downloadLimit = maxBytes ?? Config_1.config.maxResponseSize;
    const instance = axios_1.default.create({
        ...axiosConfig,
        maxContentLength: downloadLimit,
        maxBodyLength: downloadLimit,
        // Add responseType for better handling
        responseType: 'arraybuffer',
        // Don't automatically reject on error status codes
        validateStatus: () => true,
    });
    // Add request interceptor to add a cancel token
    instance.interceptors.request.use((request) => {
        const source = axios_1.default.CancelToken.source();
        request.cancelToken = source.token;
        request._cancelSource = source;
        return request;
    });
    // Add response interceptor to check size
    instance.interceptors.response.use((response) => {
        // Check size for different response types
        if (response.data) {
            let dataSize = 0;
            if (response.data instanceof ArrayBuffer) {
                dataSize = response.data.byteLength;
            }
            else if (typeof response.data === 'string') {
                dataSize = response.data.length;
            }
            else if (Buffer.isBuffer(response.data)) {
                dataSize = response.data.length;
            }
            else if (typeof response.data === 'object') {
                // For JSON responses
                dataSize = lib_types_1.Utils.safeStringify(response.data).length;
            }
            if (dataSize > downloadLimit) {
                throw new Error(`Response size of ${dataSize} bytes exceeds limit of ${downloadLimit} bytes`);
            }
        }
        // Also check Content-Length header if available
        const contentLength = parseInt(response.headers['content-length'] || '0', 10);
        if (contentLength > 0 && contentLength > downloadLimit) {
            throw new Error(`Response content length ${contentLength} bytes exceeds limit of ${downloadLimit} bytes`);
        }
        return response;
    }, (error) => {
        if (axios_1.default.isCancel(error)) {
            throw new Error(`Response size exceeds limit of ${downloadLimit} bytes`);
        }
        if (error.message &&
            (error.message.includes('maxContentLength') ||
                error.message.includes('maxBodyLength') ||
                error.message.includes('socket hang up'))) {
            throw new Error(`Response size exceeds limit of ${downloadLimit} bytes`);
        }
        throw error;
    });
    // Handle download progress
    instance.defaults.onDownloadProgress = (progressEvent) => {
        if (progressEvent.loaded > downloadLimit) {
            try {
                const event = progressEvent;
                if (event.config?._cancelSource?.cancel) {
                    event.config._cancelSource.cancel(`Response size exceeds limit of ${downloadLimit} bytes`);
                }
            }
            catch (err) {
                // Silently handle any errors with the cancellation
                console.error('Error during request cancellation:', err);
            }
        }
    };
    return instance;
}
exports.customAxios = customAxios;
//# sourceMappingURL=customHttpFunctions.js.map